%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                               %
%               EROSION SIMULATION OF A LANDSCAPE               %
%                                                               %
%                                                               %
%       John Domingo -- University of California, Berkeley      %
%           EPS 109 -- B. Militzer -- November 27, 2012         %
%                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
clf

global A ZS drain drainage pool ocean ZBeachLevel AOcean;

NX = 70; %1st index  = row index
NY = 95; %2nd index  = column index in matrix below

d  = 5; % grid spacing in meters
dx = d; % keep dx=dy for simplicity
dy = d;

LX=NX*dx;
LY=NY*dy;

% Plot View
vx = 1;
vy = -1;
vz = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                               %
%            INITIAL CONDITIONS FOR MORDOR LANSCAPE             %
%                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Z = zeros(NX,NY);

ground_level = 10;
Z(1:NX,1:NY) = ground_level;


%%%%%%%%%%%%%%%%%%%  S E A  O F  N U R N E N  %%%%%%%%%%%%%%%%%%%

 % Initial "Lake"
Z(20,40:45) = 0; Z(19:20,45:50) = 0; Z(19:21,50:55) = 0;
Z(22:25,55:60) = 0; Z(20:23,60:63) = 0; Z(23:24,64:66) = 0;
Z(22:24,67:68) = 0; Z(23:24,69:70) = 0; 

 % Westward stream
Z(13,19:24) = 0; Z(14,23:26) = 0; Z(15,26:29) = 0; Z(16,29:35) = 0;
Z(16:17,35) = 0; Z(18,35:38) = 0; Z(19,38:40) = 0;

 % Southward stream
Z(18:19,63) = 0; Z(16:18,62) = 0; Z(15:16,61) = 0; Z(15,60) = 0;
Z(12:15,59) = 0; Z(14,58) = 0; Z(11:12,60) = 0; Z(11,61:62) = 0;
Z(10,62) = 0;

 % Eastward stream
Z(24:25,71) = 0; Z(25,72:74) = 0; Z(26,74:75) = 0; Z(27,75:77) = 0;
Z(28,77:81) = 0; Z(29,81:82) = 0; Z(30:33,82) = 0; Z(33,81:82) = 0;
Z(34,80:81) = 0; Z(35,77:80) = 0; Z(36:37,77) = 0; Z(37,75:76) = 0;
Z(38,72:75) = 0; Z(37,69:72) = 0; Z(38:39,69) = 0; Z(40,69:70) = 0;
Z(40:44,71) = 0; Z(44,69:71) = 0; Z(45:47,69) = 0; Z(47,66:68) = 0;
Z(48,65:66) = 0; Z(49,62:65) = 0; Z(50,61:62) = 0;

% Northward stream
Z(26:28,60) = 0; Z(28,58:60) = 0; Z(29,57:58) = 0; Z(30,54:57) = 0;
Z(29,52:54) = 0; Z(30:31,52) = 0; Z(33,51:53) = 0; Z(31:33,51) = 0;
Z(34,52) = 0; Z(35,50:52) = 0;Z(36,49:50) = 0; Z(37,47:49) = 0;
Z(38:39,48) = 0;


% Coordinates for bounding box within which contouring iterates (see below)

nx1 = [1 15 15 15 15 15];
nx2 = [NX 30 27 27 27 27];
ny1 = [1 30 40 40 40 40];
ny2 = [NY 80 70 70 70 70];

% Contouring the Sea of Nurnen, such that the depression of the central
% lake is wider and more gradually contoured and the depression of the
% connecting streams is steeper and narrower.

% Using the coordinate vectors above, the for-loops below will iterate over
% the whole landscape from 1:NX to 1:NY and in the second pass, will only
% iterate from 15:30 to 30:80 and so on.

depression = [0 2 4 5 6 7];

for h = 1:5
    for j = ny1(h):ny2(h)
        jU = j+1;
        jD = j-1;
        for i = nx1(h):nx2(h)
            iR = i+1;
            iL = i-1;
            if Z(i,j) == depression(h)
                if Z(iR,j) == ground_level
                    Z(iR,j) = depression(h+1);
                end
                if Z(iL,j) == ground_level
                    Z(iL,j) = depression(h+1);
                end
                if Z(i,jU) == ground_level
                    Z(i,jU) = depression(h+1);
                end
                if Z(i,jD) == ground_level
                    Z(i,jD) = depression(h+1);
                end
            end
        end
    end
end



%%%%%%%%%%%%%%%%%%%  M O U N T A I N  C H A I N S  %%%%%%%%%%%%%%%%%%%

 % Colums in the followiung matrix represent
 % i0,j0,r, and h, respectively. I estimated the coordinates from
 % several fanmade topography maps I found online (see paper).
 
mordor = [65,6,1,28; 64,5,1,27; 63,4,1,28; 62,4,1,29; 61,4,1,30; 60,5,1,31;
    59,5,2,32; 58,6,2,33; 58,7,1,32; 58,8,1,33; 57,8,2,33; 57,9,2,33;
    56,7,1,32; 55,6,2,32; 53,7,3,32; 52,6,2,30; 51,7,3,32; 50,10,2,24;
    49,11,3,25; 48,11,3,24; 47,10,2,25; 46,10,4,25; 45,11,2,26; 44,12,1,27;
    43,12,1,25; 42,13,2,27; 41,13,2,26; 40,13,2,25; 39,14,1,28; 38,15,1,28;
    37,15,2,27; 36,16,3,29; 35,15,3,28; 34,17,3,27; 34,18,2,29; 33,19,2,28;
    33,20,1,27; 33,21,1,27; 33,22,1,27; 33,23,1,28; 34,24,1,29; 35,28,2,27;
    33,17,1,30; 33,16,1,30; 32,17,1,31; 31,18,1.5,32; 30,17,1.5,31;
    29,16,1.5,30; 28,15,1,29; 27,15,1,28; 26,16,1,27; 25,16,1,28; 24,14,1,29;
    23,14,2,30; 22,13,2,29; 22,15,2,29; 21,15,2,28; 21,15,1,25; 20,15,1,26;
    20,16,2,28; 19,16,2,28; 19,15,2,30; 18,15,1,29; 18,16,2,28; 18,17,2,27;
    17,17,2,28; 16,17,1,26; 11,15,1,27; 10,15,2,28; 10,16,2,28; 9,16,2,29;
    9,17,1,29; 8,18,2,30; 8,19,2,33; 7,20,1,34; 8,21,2,32; 8,22,1,30;
    7,23,1,26; 7,24,1,26; 7,25,1,28; 7,26,1,30; 8,27,1,28; 8,28,1,26;
    9,29,2,30; 8,30,1,32; 8,31,1,34; 7,32,1,35; 7,33,1,35; 8,35,1,36;
    8,36,1,30; 8,37,1,32; 7,38,1,34; 7,39,1,36; 7,40,1,38; 7,41,1,38;
    7,42,2,38; 8,43,2.5,36; 8,44,1,35; 8,45,1,33; 8,46,1.5,32; 8,47,2,34;
    9,48,1,36; 8,49,1.5,38; 8,50,1.5,38; 7,51,1,30; 7,52,1,29; 6,53,1,28;
    6,54,1,29; 7,55,1,30; 7,56,1,30; 8,57,1,30; 7,58,0.75,30; 7,59,1,26; 
    8,60,1,23; 8,61,1,23; 7,62,1,26; 7,63,1,28; 6,64,1,32; 7,65,1.5,35; 
    7,66,1.5,40; 8,67,1.75,38; 9,68,1.5,37; 10,70,1,30; 10,70,1,24;
    11,70,1,39; 11,71,1,46; 12,72,1,50; 13,73,1,42; 14,73,1,30; 15,73,1,50;
    15,74,1,44; 16,75,1,39; 17,75,1,39; 18,75,1,20; 18,76,1,25;
    19,77,1.5,17; 20,77,1,30; 20,78,1,33; 20,79,1,39 ; 22,79,1,25;
    22,80,1,28; 22,81,1,40; 22,82,1.25,35; 22,83,0.75,28; 23,82,0.5,35;
    23,83,0.75,35; 23,84,1,15; 23,85,0.6,25; 23,86,1.25,35; 23,87,1.25,25;
    23,88,1.25,15; 24,89,1,40; 24,90,0.7,35; 24,91,0.3,25; 65,7,1,30;
    65,8,1.1,31; 65,10,1.2,32; 65,11,1.3,33; 65,12,1.4,32; 64,13,1.5,32;
    64,14,1.4,33; 63,15,1.3,32; 63,16,2,32; 63,17,2,32; 58,8,1.1,35;
    58,9,1.2,35; 58,10,1.3,36; 57,11,1.4,36; 57,12,1.4,37; 57,14,1.5,37;
    57,15,1.4,38; 58,16,1.4,38; 59,17,1.3,39; 59,18,1.3,39; 60,18,1.2,30;
    60,19,1.2,30; 61,16,1.1,31; 61,17,1.1,31; 61,18,1,32; 61,19,1,32;
    60,20,1,31; 60,21,1,31; 60,22,1.1,31; 59,23,1.1,30; 59,24,1.1,30;
    59,25,1.2,29; 58,26,1.2,29; 58,27,1.3,29; 58,28,1.3,28; 57,29,1.3,28;
    57,30,1.4,27; 57,31,1.4,27; 57,32,1.4,27; 56,33,1.5,27; 56,34,1.5,28;
    56,35,1.4,28; 56,36,1.3,29; 56,37,1.3,29; 55,38,1.2,30; 55,39,1.1,30; 
    57,34,1,31; 57,35,1,32; 57,36,1,33; 58,37,1,33; 58,38,1,34;
    58,39,1.1,34; 58,40,1.2,35; 58,41,1.2,36; 59,42,1.4,36; 59,43,1.5,36;
    59,44,1.4,37; 59,45,1.3,37; 59,46,1,38; 60,47,1,38; 60,48,1,39;
    60,49,1,39; 61,50,1,40; 61,51,1,41; 61,52,1.2,41; 61,53,1.3,41;
    61,54,1.4,42; 61,55,1.2,42; 62,56,1,42; 62,57,1,43; 63,58,1.3,43;
    64,59,1.3,44; 64,60,1.1,44; 65,61,1.3,45; 64,62,1.8,45; 64,63,1,46;
    64,64,1.1,46; 64,65,1,47; 64,66,1.3,47; 64,67,1.4,46; 63,68,1.4,45;
    63,69,1.4,45; 63,70,1.5,44; 63,71,1.5,44; 63,72,1.5,43; 63,73,1.4,43;
    63,74,1.4,44; 62,75,1.4,43; 62,76,1.3,43; 61,77,1.3,42; 61,78,1.2,42;
    61,79,1.1,41; 60,80,1.1,41; 60,81,1.1,42; 60,82,1,41; 61,83,1,40;
    61,84,1.9,40;61,85,1.9,39; 38,40,1,39; 38,41,1.2,38; 39,42,1.2,37;
    40,43,1.3,36; 40,44,1.3,36; 40,45,1.4,35; 41,46,1.4,35; 42,47,1.3,35;
    42,48,1.2,34; 43,49,1,34; 43,50,1,33; 45,49,1,33; 45,50,1.2,33;
    46,51,1.2,32; 46,52,1.3,32; 46,53,1.4,31; 47,54,1.4,31; 48,55,1.4,30;
    48,56,1.3,30; 49,56,1.6,29; 50,56,1.5,29; 51,57,1.4,28; 51,58,1.3,28;
    52,58,1.2,28; 53,59,1.2,27; 55,55,1,25; 55,56,1,26; 55,57,1,28;
    56,57,1.3,30; 56,58,1.2,32; 56,59,1,30; 56,60,1.1,30; 57,60,1,28;
    57,61,1,28; 57,62,1,26; 58,62,1.2,26; 58,63,1.1,24; 59,63,1,24;
    60,63,1,22; 61,64,1,22; 61,65,1,24; 61,66,1,26; 62,67,1.2,26];

 % Plot mountain chains.

for i = 1:size(mordor,1)
    Z = initial_conditions_add_hill(Z,NX,NY,mordor(i,1),mordor(i,2),mordor(i,3),mordor(i,4));
end



%%%%%%%%%%%%%%%%%%%%%%  M O U N T  D O O M  %%%%%%%%%%%%%%%%%%%%%

Z = initial_conditions_add_hill(Z,NX,NY,52,23,1.75,100);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                               %
%                  INITIAL CONDITIONS FOR WATER                 %
%                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%set the ocean volume
oceanLevelParameter = 0.1; % what does this parameter do?
Zmin = min(min(Z));
Zmax = max(max(Z));
 %ZBeachLevel = Zmin+oceanLevelParameter*(Zmax-Zmin); 
ZBeachLevel = 10;
VOcean=0.0;
AOcean=0;
for i = 1:NX
    for j = 1:NY
        if (Z(i,j)<=ZBeachLevel)
            AOcean = AOcean+1;
            VOcean = VOcean+ZBeachLevel-Z(i,j);
        end
    end
end
fprintf('Minimum elevation      %f\n',Zmin);
fprintf('Maximum elevation      %f\n',Zmax);
fprintf('Beach level            %f\n',ZBeachLevel);
fprintf('Ocean volume           %f\n',VOcean);
fprintf('Ocean surface fraction %f %%\n\n',AOcean/(NX*NY)*100);

%time step
dt = 50;  % dt<dx^2/(4*D) not just dx^2/(2*D)

%Area exponent, setting m=1
m=1;

%gradient exponent, setting n=1
n=1;

%threshold 
theta_c = 1;

K = 1e-6; % meters^(1-2m)/yr

D = 0.0005; % m^2/yr
% D = 0.0;

% any uplift?
U = 0.00;

eta1 = D*dt/dx/dx;
eta2 = D*dt/dy/dy;
if (eta1>=0.25 || eta2>=0.25)
    fprintf('Check if both eta values are below 0.25: %f %f\n',eta1,eta2); 
    pause
end

fprintf('erosion_lab8: Stardard simulation n=%.2f m=%.2f and small dt=%.0f\n\n',n,m,dt);

x = 1:NX;
y = 1:NY;
[X,Y] = meshgrid(y,x); %strange the y goes first !!!

pool_check10(Z,NX,NY,VOcean)

subplot(1,2,1);
pcolor(X,Y,Z);
surf(X,Y,Z);
axis([0 NY 0 NX 0 100]);
title('Surface Relief');
xlabel('x');
ylabel('y');
view([vx vy vz]);
subplot(1,2,2);
pne = zeros(NX,NY); %pne = not-a-pool(NX,NY)
pne(pool==0) = 1;
pcolor(X,Y,pne);
colormap jet;
title('Pool Locations');
xlabel('x');
ylabel('y');
set(figure(1), 'position', [10, 200, 1200, 400], 'PaperPositionMode', 'auto','color', 'white')
drawnow;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                               %
%                       EROSION OF MORDOR                       %
%                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

A = zeros(NX,NY);

Znew = Z;
jt = 1;
f = 0;
t = 0; %time in years
for it = 1:2000*(600/dt)
    t=t+dt;
    
    calculate_collection_area2(Z,NX,NY);
    A = A.*(dx*dy);
    
    for i = 1:NX
        iL = mod(i-1-1,NX)+1; % normally i-1 but observe p.b.c.
        iR = mod(i+1-1,NX)+1; % normally i+1 but observe p.b.c.

        for j = 1:NY
            jD = mod(j-1-1,NY)+1; % normally j-1 but observe p.b.c.
            jU = mod(j+1-1,NY)+1; % normally j+1 but observe p.b.c.

            uplift = U;
            if ocean(i,j)
                Gz     = 0;
                phiz   = 0;
                uplift = 0;
            elseif drain(i,j)>0 %this cell is a drain
                s1 = (Z(iR,j)  - Z(iL,j))/(2*dx);
                s2 = (Z(i,jU)  - Z(i,jD))/(2*dy);
                s3 = (Z(iR,jD) - Z(iL,jU))/(2*sqrt(dx^2+dy^2));
                s4 = (Z(iR,jU) - Z(iL,jD))/(2*sqrt(dx^2+dy^2));
                gradient = ( sqrt(s1^2 + s2^2) + sqrt(s3^2 + s4^2) )/2;
                
                Gz = K*( A(i,j)^m*gradient^n - theta_c );                  
                % diffusion term
                phiz = D*((Z(iL,j) - 2*Z(i,j) + Z(iR,j))/(dx*dx) + (Z(i,jU) - 2*Z(i,j) + Z(i,jD))/(dy*dy));

            elseif drainage(i,j)>0 %this cell is a drainage point (it drains a pool)
                
                if (Z(i,j)>=Z(iR,j)) && pool(iR,j)~=drainage(i,j) 
                    gradient = (Z(i,j)-Z(iR,j))/dx; %pool is on my left, I drain to the right, use this gradiant
                elseif (Z(i,j)>=Z(iL,j)) && pool(iL,j)~=drainage(i,j)
                    gradient = (Z(i,j)-Z(iL,j))/dx;
                elseif (Z(i,j)>=Z(i,jU)) && pool(i,jU)~=drainage(i,j)
                    gradient = (Z(i,j)-Z(i,jU))/dy;
                elseif (Z(i,j)>=Z(i,jD)) && pool(i,jD)~=drainage(i,j)
                    gradient = (Z(i,j)-Z(i,jD))/dy;
                else
                    error('error',i,j); % should not happen
                end
                
                Gz = K*( A(i,j)^m*gradient^n - theta_c);                
                % diffusion term as before
                phiz = D*((Z(iL,j) - 2*Z(i,j) + Z(iR,j))/(dx*dx) + (Z(i,jU) - 2*Z(i,j) + Z(i,jD))/(dy*dy));

            else %this cell is a pool, assume it has the regular mass diffusion but no erosion!
                Gz = 0;
                phiz = D*((Z(iL,j) - 2*Z(i,j) + Z(iR,j))/(dx*dx) + (Z(i,jU) - 2*Z(i,j) + Z(i,jD))/(dy*dy));
            end
            
            if (Gz<0) Gz = 0; end % no erosion if A^m * g^n < theta_c

            Znew(i,j) = Z(i,j) + dt*(phiz - Gz) + uplift*(dt/600);  % add some uplift
            
            if (Znew(i,j)<0)
                Znew(i,j) = 0;  % ???? yes this does happen at the boundary when kept at zero
            end            
        end
    end
    
    Z = Znew;
    pool_check10(Z,NX,NY,VOcean)
        
    if (it == 1 | mod(it,round(jt))==0)
        if it>jt 
            jt = jt*1.1;
        end
        if pool(:,:) pool=1; end
        fprintf('Iteration=%5d  time= %6d  ocean level=%.4f   ocean surface fraction=%.3f\n',it,t,ZBeachLevel,100*AOcean/(NX*NY));

        subplot(1,2,1);
       
        pcolor(X,Y,Z);
        f = f+1;
        colormap('jet');
        surf(X,Y,Z)
        title('Surface Relief')
        xlabel('x');
        ylabel('y');
        view([vx+(f*0.05), vy+(f*0.05), vz+(f*0.05)]);
        axis([0 NY 0 NX 0 150]);
        saveas(gcf,['Mordor', sprintf('%03d',f),'.png'])
        pause(0.01)

        subplot(1,2,2);

        hold off
        pcolor(X,Y,Z)
        shading interp
        hold on
        contour(X,Y,Z,30,'k');  

        drawnow;
        
    end
end % end of main loop over iterations

%final plot
subplot(1,2,1);
hold off
surf(X,Y,Z);
title('Surface Relief')
xlabel('x');
ylabel('y');
view([vx vy vz]);
axis([0 NY 0 NX 0 150]);

subplot(1,2,2);
hold off
pcolor(X,Y,Z)
shading interp
hold on
contour(X,Y,Z,30,'k');  
axis([0 NY 0 NX 0 150]);

fprintf('Simulation finished.\n\n');
